/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.componentmanager;

//---IMPORTS---
import java.util.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;

/**
 * A Configuration Proxy hold configurations caches and
 * maintains configurations obtained from the Configuration
 * Manager.
 *
 * It also has support for checking for updates in configurations,
 * if a change is detected, it reloads all the changed configurations.
 * This is done by a thread which runs every ConfigurationProxy.updateTime
 * seconds.
 *
 * This is a singleton class, and should be used by all classes
 * that wish to gain a configuration from the server, as it then
 * allows them to be dynamically reconfigured.
 *
 * @author  $Author: tdb $
 * @version $Id: ConfigurationProxy.java,v 1.19 2004/08/01 10:40:50 tdb Exp $
 */
public class ConfigurationProxy extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.19 $";
    
    /**
     * The default time to refresh the
     * configurations - 60 seconds.
     */
    public final int DEFAULT_REFRESH_TIME = 60;

//---STATIC METHODS---

    /**
     * Return a reference to the single class.
     * Construct it if it does not already exist, otherwise just return the reference.
     */
    public synchronized static ConfigurationProxy getInstance() {
        if (_instance == null){
            _instance = new ConfigurationProxy();
        }
        return _instance;
    }

//---CONSTRUCTORS---
    
    /**
     * Construct a ConfigurationProxy
     * Then starts the auto-updating part (the thread) running.
     */
    private ConfigurationProxy() {
        // set the Thread name
        setName("componentmanager.ConfigurationProxy");
        start();    
    }

//---PUBLIC METHODS---

    /**
     * This method runs and waits for a period defined by
     * ConfigurationProxy.updateTime.  Each time it checks all the 
     * configuratons it has loaded to see if their configurations have
     * changed.  If they have, it refreshes the cache with the new configuration
     * and then instructs the old one to be disconnected from the ORB so that it
     * can be garbage collected.
     */
    public void run() {
        // periodic checks for config changes here
        
        while(_running) {
            int refreshTime = 0;
            try {
                refreshTime = Integer.parseInt(getProperty("ConfigurationProxy", "ConfigurationProxy.updateTime"));
            } catch (NumberFormatException e) {
                refreshTime = DEFAULT_REFRESH_TIME;
                _logger.write(toString(), Logger.WARNING, "Erronous ConfigurationProxy.updateTime value in configuration using default of " + refreshTime + " seconds");
            } catch (PropertyNotFoundException e) {
                refreshTime = DEFAULT_REFRESH_TIME;
                _logger.write(toString(), Logger.WARNING, "ConfigurationProxy.updateTime value unavailable using default of " + refreshTime + " seconds");
            }
        
            try {
                Thread.sleep(refreshTime * 1000);
            } catch (InterruptedException e) {
            }
            // ensures we update the list independently of the getProperty method
            synchronized (this) {
                Iterator i = _configCache.keySet().iterator();
                while(i.hasNext()) {
                    String configName = (String) i.next();
                    ConfigurationCache cc = (ConfigurationCache) _configCache.get(configName);
                    if (_confman.isModified(cc.getConfig().getFileList(), cc.getConfig().getLastModified())) {
                        // disconnect the old Configuration object from the ORB
                        cc.getConfig().disconnect();
                        // get the updated Configuration from the manager
                        cc.setConfig(_confman.getConfiguration(configName));
                        cc.setPropertyCache(new HashMap());
                    }
                }
            }
        }
    }

    /**
     * This method obtains the configuration from the configuration
     * cache, if the configuration isn't in the cache, it adds it.
     * It then asks the configuration for the requested property
     * name, and then returns that to the caller.
     *
     * @param configName the name of the configuration
     * @param propertyName the name of the property to obtain
     *
     * @return the value of the requested property
     */
    public String getProperty(String configName, String propertyName) throws PropertyNotFoundException {
        // ensures the run method can update the cache if needed.
        synchronized (this) {
            if (!_configCache.containsKey(configName)) {
                _configCache.put(configName, new ConfigurationCache(_confman.getConfiguration(configName), new HashMap()));
            }
            ConfigurationCache cc = (ConfigurationCache) _configCache.get(configName);
            HashMap propertyCache = cc.getPropertyCache();
            Configuration config = cc.getConfig();
            if(!propertyCache.containsKey(propertyName)) {
                // try and get the config 
                // if we can't throw an exception
                try {
                    propertyCache.put(propertyName, config.getProperty(propertyName));
                } catch (org.omg.CORBA.MARSHAL e) {
                    // IMPORTANT - make sure we cache the fact something doesn't exist
                    propertyCache.put(propertyName, null);    
                }
            }
            String property = (String) propertyCache.get(propertyName);
            if(property == null) {
                // the property cannot be found, so we throw back
                // a PropertyNotFoundException and give up.
                throw new PropertyNotFoundException("Could not locate the property \""+propertyName+"\" in the configuration \""+configName+"\"");
            }
            return property;
        }
    }
    
    /**
     * This method obtains the configuration from the configuration
     * cache, if the configuration isn't in the cache, it adds it.
     * It then returns the file list used to build that configuration.
     *
     * @param configName the name of the configuration
     *
     * @return the semi-coma separated list of filenames
     */
    public String getFileList(String configName) {
        // ensures the run method can update the cache if needed.
        synchronized (this) {
            if (!_configCache.containsKey(configName)) {
                _configCache.put(configName, new ConfigurationCache(_confman.getConfiguration(configName), new HashMap()));
            }
            ConfigurationCache cc = (ConfigurationCache) _configCache.get(configName);
            return cc.getConfig().getFileList();
        }
    }
    
    /**
     * This method obtains the configuration from the configuration
     * cache, if the configuration isn't in the cache, it adds it.
     * It then returns the time in milliseconds of the most last
     * modified date of the most recently modified file in the configurations
     * file list. 
     *
     * @param configName the name of the configuration
     *
     * @return the time in millis
     */
    public long getLastModified(String configName) {
        // ensures the run method can update the cache if needed.
        synchronized (this) {
            if (!_configCache.containsKey(configName)) {
                _configCache.put(configName, new ConfigurationCache(_confman.getConfiguration(configName), new HashMap()));
            }
            ConfigurationCache cc = (ConfigurationCache) _configCache.get(configName);
            return cc.getConfig().getLastModified();
        }
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * This holds a reference to the configuration
     * manager
     */
    private ConfigurationManager _confman = ReferenceManager.getInstance().getCM();
    
    /**
     * Holds a list of configuration cache's for every configuration that
     * has been requested of this proxy.
     */
    private HashMap _configCache = new HashMap();
    
    /**
     * Holds the current state of the configuration proxy's thread
     */
    private boolean _running = true;

//---STATIC ATTRIBUTES---

    /**
     * A reference to the single instance of this class
     */
    private static ConfigurationProxy _instance;

}
