/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.core.loggers;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.core.*;
import java.util.Date;
import java.text.DateFormat;
import java.util.Locale;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

/**
 * The FileLogger is an implementation of the LoggerImpl defined
 * in the associated interface. It provides quite extensive features to allow
 * logging ot a text file.
 *
 * @author  $Author: tdb $
 * @version $Id: FileLogger.java,v 1.13 2004/08/01 10:40:56 tdb Exp $
 */
public class FileLogger implements LoggerImpl {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.13 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

	/**
	 * Creates a new FileLogger.
	 *
	 * The constructor checks that writing will be ok, then sets up the
	 * writer. It makes use of the fileCheck() method found later on to
	 * ensure writing will be ok, and throws an IOException if there is
	 * a problem. Errors at this stage cannot be logged because the
	 * logging mechanism has not be setup completely, so any problems
	 * are printed to the screen in the same format used for logging.
	 * 
	 * @throws IOException if there is a problem with the file check.
	 */
    public FileLogger() throws IOException{
        filename = System.getProperty("uk.org.iscream.cms.server.LoggerClass.FileLogger.filename");
		// Perform file check to make sure writing is ok
        if(!fileCheck()){
			// Have to system.out.println errors because logging mechanism failed !
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "File check failed, construction terminated"));
            throw new IOException("File check failed, unable to create FileLog");
        }
        try{
			// Setup the writer
            writer = new BufferedWriter(new FileWriter(filename, true));
			// File is now open
			open = true;
        }
        catch(IOException e){
            System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Attempt to setup writer failed - " + e.getMessage()));
        }
		// Put an initial line into the log
        write(toString(), Logger.SYSINIT, "started");
    }

//---PUBLIC METHODS---
    
    /**
	 * The write() method takes a line of text, pre-formatted
	 * and outputs it using a method defined by the actual
	 * implementation. The verbosity is given in case the
	 * implementation wishes to utilise it in the layout -
	 * eg. a different colour or font.
	 * 
	 * This instance writes the line to a file.
	 *
	 * @param line A line of formatted text to be logged
	 * @param verbosity the verbosity of this message
	 */  
    public synchronized void write(String line, int verbosity) {
		// Check to make sure file is open
		if(open){
			// Produce a nicely formatted line for the logfile
			try{
				// We have to synchronize here due to problems with two write()'s being called before a newLine()
				synchronized(writer){
					// Attempt to write the line
					writer.write(line);
					// Best to use newLine() as it will use the correct platform encoding
					writer.newLine();
				}
				// Make sure the line is written immeidiately
				writer.flush();
			}
			catch(IOException e){
				// We'd best log the error
				System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Writing to logfile failed - " + e.getMessage()));
				// As it's an IOException we should suspend logging
				open = false;
	        }
		}
		else{
			// If file is not open we should print this to the screen
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Write failed - file not open"));
		}
    }
 
	/**
	 * The clear() method erases the contents of the logfile. It does
	 * this in a very clumsy way - by closing the writer and opening a
	 * new one which is set not to append. This results in the logfile
	 * being overwritten, which effectively erases it.
	 */
    public void clear(){
		// Check file is open
		if(open){
			write(toString(), Logger.SYSMSG, "Attempting to clear logfile");
			try{
				// Close appending writer
				open = false;
				writer.close();
				// Open a non-appending writer
				writer = new BufferedWriter(new FileWriter(filename));
				open = true;
			}
			catch(IOException e){
				// We'd best log the error
				write(toString(), Logger.ERROR, "Attempt to clear logfile failed - "+e.getMessage());
				// As it's an IOException we should suspend logging
				open = false;
			}
		}
		else{
			// If file is not open we should print this to the screen
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Clearing failed - file not open"));
		}
    }
    
	/**
	 * The close() method closes the logfile and no more data can be written.
	 * If writing is attempted after close() has been called an error will be
	 * generated. The open() method can be used to reopen or open a new file.
	 */
    public void close(){
		// Check to see if a file is open
		if(open){
			write(toString(), Logger.SYSMSG, "Attempting to terminate logging");
			try{
				open = false;
		        writer.close();		
			}
			catch(IOException e){
				write(toString(), Logger.ERROR, "Attempt to close logfile failed - " + e.getMessage());
				// As it's an IOException we should suspend logging
				open = false;
			}
		}
		else{
			// If file is not open we should print this to the screen
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Close failed - file not open"));
		}
    }
    
	/**
	 * The open() method opens a new file after the close() method has
	 * been called on the FileLog. This enables the user to easily change
	 * the file to which the log is being written. An error is generated
	 * if a file is already open.
	 * 
	 * @param filename The new file to write to.
	 * @throws IOException if the file cannot be written to.
	 */
	public void open(String filename) throws IOException{
		// Check to see if a file is open
		if(open){
			// If file is already open then we should log that this went wrong
			write(this.toString(), Logger.WARNING, "Open failed - a file is already open");
		}
		else{
			this.filename = filename;
			if(!fileCheck()) {
				// Have to system.out.println errors because logging mechanism failed !
				System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "File check failed, construction terminated"));
				throw new IOException("File check failed, unable to create FileLog");
			}
			try{
				// Setup the writer
				writer = new BufferedWriter(new FileWriter(filename, true));
				// File is now open
				open = true;
			}
			catch(IOException e){
			    System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Attempt to open writer failed - " + e.getMessage()));
			}
		}
	}

	/**
	 * The suspend() method halts writing to the file by setting the open
	 * boolean to false, and thus blocking the write() method. A line is
	 * written to the logfile beforehand to note that writing has been
	 * suspended.
	 */
	public void suspend(){
		// Check to see if a file is open
		if(open){
			// Make a note of the fact that writing has been suspended
			write(toString(), Logger.SYSMSG, "Writing suspended");
			// Make sure writing not permitted
			open=false;
		}
		else{
			// If file is not open we should print this to the screen
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Suspend failed - file not open"));
		}
	}
	
	/**
	 * The resume() method resumes writing to the file by setting the open
	 * boolean to true. A line is written to the logfile to signify this
	 * event occuring.
	 */
	public void resume(){
		// Check to see if a file is open
		if(open){
			// If file is open we should print this to the screen
			System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "Resume failed - file open"));
		}
		else{
			// Permit writing again
			open=true;
			// Make a note of the fact that writing has been resumed
			write(toString(), Logger.SYSMSG, "Writing resumed");
		}
	}
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }
    
//---PRIVATE METHODS---

	/**
	 * The fileCheck() method is used to ensure that writing is ok. It
	 * performs so basic checks to make sure that if the file exists
	 * it can be written to, and if not a new file is created. A boolean
	 * result is returned to signify success or failure.
	 * 
	 * @return A boolean value signify whether the test was sucessful.
	 */
    private boolean fileCheck(){
        boolean fileOK = false;
        try{
            File file = new File(filename);
			
			// File already exists
            if(file.exists()) {
                /* Only want success if it is a file (not a directory)
				 * and can be written to.
				 */
				if(file.isFile() && file.canWrite()) {
                    fileOK = true;
                }
            }
			// Create a new file and suceed
            else{
                file.createNewFile();
                fileOK = true;
            }
        }
        catch(IOException e){
            System.out.println(FormatName.formatLogLine(this.toString(), Logger.FATAL, "File check failed - "+e.getMessage()));
        }
        
        return fileOK;
    }

    /**
     * This method is provided if this class wishes to log
     * a message itself.
     *
	 * @param source A String representation of the source
	 * @param verbosity the verbosity of this message
	 * @param message The message to log
	 */  
    private void write(String source, int verbosity, String message) {
        write(FormatName.formatLogLine(source, verbosity, message), verbosity);
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

	/**
	 * The filename of the currently open, or last open, file.
	 */
    private String filename;
	
	/**
	 * A reference to the writer being used.
	 */
    private BufferedWriter writer;
	
	/**
	 * A boolean signifying whether a file is open or not.
	 */
	private boolean open = false;
	
	/**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = Core.NAME;

//---STATIC ATTRIBUTES---
    
}
