/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filter;

//---IMPORTS---
import java.io.*;
import java.net.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.filter.*;
import uk.org.iscream.cms.util.*;

/**
 * This class contains the main method to be run by
 * the filterd.  It harvests UDP traffic, and queues it.
 *
 * @author  $Author: tdb $
 * @version $Id: UDPReader.java,v 1.27 2005/09/25 09:57:42 tdb Exp $
 */
public class UDPReader extends Thread{

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.27 $";
    
    /**
     * The maximum size of a packet
     */
    private final int packetSizeLimit = 8192;   
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Constructs a new UDPReader.
     *
     * @param port The port on which we listen for UDP data
     * @param queue The queue which we are using
     */
    public UDPReader(int port, Queue queue){
        // set the Thread name
        setName("filter.UDPReader");
        
        _port = port;
        _queue = queue;
        _logger.write(toString(), Logger.SYSINIT, "started");
    }

//---PUBLIC METHODS---
    
    /**
     * The main method in the class. Reads and queues XML sent
     * over UDP.
     */
    public void run() {
        // setup an empty ACL defaulting to ALLOW
        ACL acl = new ACL(ACL.ALLOW);
        
        // setup a Datagram socket
        DatagramSocket socket = null;
        try {
            socket = new ACLDatagramSocket(acl, _port);
        }
        catch (BindException e){
            _logger.write(this.toString(), Logger.FATAL, "Could not start the UDPReader thread on port "+_port+" as this port was already in use.");
            return;
        }
        catch (Exception e){
            _logger.write(this.toString(), Logger.FATAL, "Could not start the UDPReader thread on port "+_port+".");
            return;
        }
        
        _logger.write(this.toString(), Logger.SYSMSG, "UDPReader thread ready and listening for UDP packets on port "+_port);
        
        byte[] buf;
        
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String stringACL = "";
        String newStringACL = "";
        
        // read UDP packets and queue them
        boolean running = true;
        while (running){
            // get hold of the ACL in the configuration
            try {
                newStringACL = cp.getProperty("Filter." + FilterMain.NAME, "Filter.UDPACL");
            }
            catch(PropertyNotFoundException e) {
                // if we can't find it, we'll just use a null ACL
                newStringACL = "";
                _logger.write(toString(), Logger.WARNING, "No ACL found for UDPReader, using empty ACL instead: " + e);
            }
            // check to see if the ACL has changed
            if(!newStringACL.equals(stringACL)) {
                _logger.write(toString(), Logger.SYSMSG, "Reloading Access Control List");
                // clear the ACL
                acl.clear();
                // set the default to something sane
                acl.setDefaultMode(ACL.ALLOW);
                // add the new ACL (this may change the default)
                acl.add(newStringACL);
                stringACL = newStringACL;
            }
            try {
                
                // receive request and put it in the Queue               
                buf = new byte[packetSizeLimit];
                DatagramPacket packet = new DatagramPacket(buf, buf.length);
                socket.receive(packet);
                String xml = new String(packet.getData());
                _queue.add(xml);
            }
            catch (IOException e) {
                _logger.write(this.toString(), Logger.WARNING, "This UDPReader thread has been shut down as an exception occured: "+e);
                running = false;
            }
        }
        socket.close();
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterMain.NAME;
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * The port that this reader is using
     */
    int _port;
    
    /**
     * The Queue object
     */
    Queue _queue;

//---STATIC ATTRIBUTES---

}
