/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filtermanager;

//---IMPORTS---
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.filter.*;
import uk.org.iscream.cms.util.*;
import java.net.*;
import java.io.*;

/**
 * A socket listener to listen for new hosts registering with the system.
 * When a host makes a connection, the connecton is past to an instance
 * of the HostInit class, which handles further communication.
 *
 * @author  $Author: tdb $
 * @version $Id: HostListener.java,v 1.16 2004/08/01 10:41:05 tdb Exp $
 */
class HostListener extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.16 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

	/**
     * Constructs a new listener
     *
	 * @param listenPort The port that the server will listen on.
	 */
    public HostListener() {
        // set the Thread name
        setName("filtermanager.HostListener");
        
        _logger.write(toString(), Logger.SYSINIT, "started");
    }

//---PUBLIC METHODS---

	/**
	 * The run() method is the main loop for this thread, and we
	 * will remain in here until such a point as something goes
	 * wrong with the listening. After initially setting up the
	 * ServerSocket we go round a while loop receiving connections
	 * and then passing them off to other processes to deal with.
	 */
    public void run(){
        // setup an empty ACL defaulting to ALLOW
        ACL acl = new ACL(ACL.ALLOW);
        
        ServerSocket listenSocket=null;
		// We use this boolean so we can break out of the while loop if we want
        boolean run = true;
        try{
            // Work out the port we want
            int listenPort = Integer.parseInt(ConfigurationProxy.getInstance().getProperty("FilterManager." + FilterManager.NAME, "FilterManager.listenPort"));
			// Setup the ServerSocket so that clients can connect
			listenSocket = new ACLServerSocket(acl, listenPort);
            _logger.write(toString(), Logger.SYSMSG, "Server listening on "
										+InetAddress.getLocalHost().getHostName()
										+"/"+InetAddress.getLocalHost().getHostAddress()
										+" port "+listenSocket.getLocalPort());
            
        }
        catch(UnknownHostException e){
			_logger.write(toString(), Logger.SYSMSG, "Server listening on UnknownHost "
										+"port "+listenSocket.getLocalPort());
		}
        catch(IOException e){
            _logger.write(toString(), Logger.FATAL, "IO Error, can't start FilterManager :"+e);
            run = false;
        }
        catch(PropertyNotFoundException e){
            _logger.write(toString(), Logger.FATAL, "Fatal Error, can't find config :"+e);
            run = false;
        }
        catch(NumberFormatException e){
            _logger.write(toString(), Logger.FATAL, "Invalid port configuration found :"+e);
            run = false;
        }

		// Loop round constantly until we decide to stop
		ConfigurationProxy cp = ConfigurationProxy.getInstance();
        String stringACL = "";
        String newStringACL = "";
        while(run){
            // get hold of the ACL in the configuration
            try {
                newStringACL = cp.getProperty("FilterManager." + FilterManager.NAME, "FilterManager.ACL");
            }
            catch(PropertyNotFoundException e) {
                // if we can't find it, we'll just use a null ACL
                newStringACL = "";
                _logger.write(toString(), Logger.WARNING, "No ACL found for FilterManager, using empty ACL instead: " + e);
            }
            // check to see if the ACL has changed
            if(!newStringACL.equals(stringACL)) {
                _logger.write(toString(), Logger.SYSMSG, "Reloading Access Control List");
                // clear the ACL
                acl.clear();
                // set the default to something sane
                acl.setDefaultMode(ACL.ALLOW);
                // add the new ACL (this may change the default)
                acl.add(newStringACL);
                stringACL = newStringACL;
            }
       	    Socket hostSocket=null;
            try{
                _logger.write(toString(), Logger.SYSMSG, "Waiting for Connection");
				// This will block until a host connects - at which point we get a Socket
                hostSocket = listenSocket.accept();
                _logger.write(toString(), Logger.SYSMSG, "Connection accepted from: " + hostSocket.toString());
            }
            catch(IOException e){
				// Something went wrong with the ServerSocket, so we'll stop listening
                run=false;
            }
			// If we've stopped on the line above we won't want to try this !
            if(run){
				try {
				    // Setup the HostInit so it can carry on communications with the host
                    HostInit init = new HostInit(hostSocket);
				    // and start it
                    init.start();
                } catch (IOException e) {
                    _logger.write(toString(), Logger.ERROR, e.toString());
                }
            }
        }
		// Best log the fact that we're stopping
        _logger.write(toString(), Logger.FATAL, "Fatal error, shutdown pending");
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            this.getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
    
    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterManager.NAME;
    
//---STATIC ATTRIBUTES---

}
